@@ -4,7 +4,7 @@ void foo() { }
 
 int main(int argc, char **argv) {
   int a;
-  #pragma omp target data // expected-error {{expected at least one map clause for '#pragma omp target data'}}
+  #pragma omp target data // expected-error {{expected at least one 'map' or 'use_device_ptr' clause for '#pragma omp target data'}}
   {}
   L1:
     foo();
